@@ -15,7 +15,6 @@ ACTIONS:
                     tweets from that list
  mylist         get list of your lists; give a list name to get tweets
                     from that list
- public         get latest public tweets
  pyprompt       start a Python prompt for interacting with the twitter
                     object directly
  replies        get latest replies to you
@@ -44,6 +43,7 @@ FORMATS for the --format option
 
  default         one line per status
  verbose         multiple lines per status, more verbose status info
+ json            raw json data from the api on each line
  urls            nothing but URLs
  ansi            ansi colour (rainbow mode)
 
@@ -66,21 +66,22 @@ from __future__ import print_function
 
 try:
     input = __builtins__['raw_input']
-except AttributeError:
+except (AttributeError, KeyError):
     pass
 
 
-CONSUMER_KEY='uS6hO2sV6tDKIOeVjhnFnQ'
-CONSUMER_SECRET='MEYTOS97VvlHX7K1rwHPEqVpTSqZ71HtvoK4sVuYk'
+CONSUMER_KEY = 'uS6hO2sV6tDKIOeVjhnFnQ'
+CONSUMER_SECRET = 'MEYTOS97VvlHX7K1rwHPEqVpTSqZ71HtvoK4sVuYk'
 
-import sys
-import time
 from getopt import gnu_getopt as getopt, GetoptError
 from getpass import getpass
-import re
-import os.path
+import json
 import locale
+import os.path
+import re
 import string
+import sys
+import time
 
 try:
     from ConfigParser import SafeConfigParser
@@ -91,6 +92,11 @@ try:
     from urllib.parse import quote
 except ImportError:
     from urllib2 import quote
+try:
+    import HTMLParser
+except ImportError:
+    import html.parser as HTMLParser
+
 import webbrowser
 
 from .api import Twitter, TwitterError
@@ -105,23 +111,31 @@ OPTIONS = {
     'refresh_rate': 600,
     'format': 'default',
     'prompt': '[cyan]twitter[R]> ',
-    'config_filename': os.environ.get('HOME', '') + os.sep + '.twitter',
-    'oauth_filename': os.environ.get('HOME', '') + os.sep + '.twitter_oauth',
+    'config_filename': os.environ.get('HOME', os.environ.get('USERPROFILE', '')) + os.sep + '.twitter',
+    'oauth_filename': os.environ.get('HOME', os.environ.get('USERPROFILE', '')) + os.sep + '.twitter_oauth',
     'length': 20,
     'timestamp': False,
     'datestamp': False,
     'extra_args': [],
     'secure': True,
+    'invert_split': False,
+    'force-ansi': False,
 }
 
+gHtmlParser = HTMLParser.HTMLParser()
+hashtagRe = re.compile(r'(?P<hashtag>#\S+)')
+profileRe = re.compile(r'(?P<profile>\@\S+)')
+ansiFormatter = ansi.AnsiCmd(False)
+
 def parse_args(args, options):
     long_opts = ['help', 'format=', 'refresh', 'oauth=',
                  'refresh-rate=', 'config=', 'length=', 'timestamp',
-                 'datestamp', 'no-ssl']
+                 'datestamp', 'no-ssl', 'force-ansi']
     short_opts = "e:p:f:h?rR:c:l:td"
     opts, extra_args = getopt(args, short_opts, long_opts)
-    extra_args = [arg.decode(locale.getpreferredencoding())
-                  for arg in extra_args]
+    if extra_args and hasattr(extra_args[0], 'decode'):
+        extra_args = [arg.decode(locale.getpreferredencoding())
+                      for arg in extra_args]
 
     for opt, arg in opts:
         if opt in ('-f', '--format'):
@@ -144,6 +158,8 @@ def parse_args(args, options):
             options['secure'] = False
         elif opt == '--oauth':
             options['oauth_filename'] = arg
+        elif opt == '--force-ansi':
+            options['force-ansi'] = True
 
     if extra_args and not ('action' in options and options['action'] == 'help'):
         options['action'] = extra_args[0]
@@ -167,11 +183,33 @@ def get_time_string(status, options, format="%a %b %d %H:%M:%S +0000 %Y"):
         return time.strftime("%Y-%m-%d ", t)
     return ""
 
+def reRepl(m):
+    ansiTypes = {
+          'clear':   ansiFormatter.cmdReset(),
+          'hashtag': ansiFormatter.cmdBold(),
+          'profile': ansiFormatter.cmdUnderline(),
+          }
+
+    s = None
+    try:
+        mkey = m.lastgroup
+        if m.group(mkey):
+            s = '%s%s%s' % (ansiTypes[mkey], m.group(mkey), ansiTypes['clear'])
+    except IndexError:
+        pass
+    return s
+
+def replaceInStatus(status):
+    txt = gHtmlParser.unescape(status)
+    txt = re.sub(hashtagRe, reRepl, txt)
+    txt = re.sub(profileRe, reRepl, txt)
+    return txt
+
 class StatusFormatter(object):
     def __call__(self, status, options):
-        return ("%s%s %s" %(
+        return ("%s%s %s" % (
             get_time_string(status, options),
-            status['user']['screen_name'], status['text']))
+            status['user']['screen_name'], gHtmlParser.unescape(status['text'])))
 
 class AnsiStatusFormatter(object):
     def __init__(self):
@@ -179,18 +217,23 @@ class AnsiStatusFormatter(object):
 
     def __call__(self, status, options):
         colour = self._colourMap.colourFor(status['user']['screen_name'])
-        return ("%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" % (
             get_time_string(status, options),
-            ansi.cmdColour(colour), status['user']['screen_name'],
-            ansi.cmdReset(), status['text']))
+            ansiFormatter.cmdColour(colour), status['user']['screen_name'],
+            ansiFormatter.cmdReset(), replaceInStatus(status['text'])))
 
 class VerboseStatusFormatter(object):
     def __call__(self, status, options):
-        return ("-- %s (%s) on %s\n%s\n" %(
+        return ("-- %s (%s) on %s\n%s\n" % (
             status['user']['screen_name'],
             status['user']['location'],
             status['created_at'],
-            status['text']))
+            gHtmlParser.unescape(status['text'])))
+
+class JSONStatusFormatter(object):
+    def __call__(self, status, options):
+         status['text'] = gHtmlParser.unescape(status['text'])
+         return json.dumps(status)
 
 class URLStatusFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
@@ -218,18 +261,18 @@ class AnsiListsFormatter(object):
 
     def __call__(self, list):
         colour = self._colourMap.colourFor(list['name'])
-        return ("%s%-15s%s %s" %(
-            ansi.cmdColour(colour), list['name'],
-            ansi.cmdReset(), list['description']))
+        return ("%s%-15s%s %s" % (
+            ansiFormatter.cmdColour(colour), list['name'],
+            ansiFormatter.cmdReset(), list['description']))
 
 
 class AdminFormatter(object):
     def __call__(self, action, user):
-        user_str = "%s (%s)" %(user['screen_name'], user['name'])
+        user_str = "%s (%s)" % (user['screen_name'], user['name'])
         if action == "follow":
-            return "You are now following %s.\n" %(user_str)
+            return "You are now following %s.\n" % (user_str)
         else:
-            return "You are no longer following %s.\n" %(user_str)
+            return "You are no longer following %s.\n" % (user_str)
 
 class VerboseAdminFormatter(object):
     def __call__(self, action, user):
@@ -241,12 +284,12 @@ class VerboseAdminFormatter(object):
 
 class SearchFormatter(object):
     def __call__(self, result, options):
-        return("%s%s %s" %(
+        return("%s%s %s" % (
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
             result['from_user'], result['text']))
 
 class VerboseSearchFormatter(SearchFormatter):
-    pass #Default to the regular one
+    pass  # Default to the regular one
 
 class URLSearchFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
@@ -260,10 +303,10 @@ class AnsiSearchFormatter(object):
 
     def __call__(self, result, options):
         colour = self._colourMap.colourFor(result['from_user'])
-        return ("%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" % (
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
-            ansi.cmdColour(colour), result['from_user'],
-            ansi.cmdReset(), result['text']))
+            ansiFormatter.cmdColour(colour), result['from_user'],
+            ansiFormatter.cmdReset(), result['text']))
 
 _term_encoding = None
 def get_term_encoding():
@@ -280,6 +323,7 @@ formatters = {}
 status_formatters = {
     'default': StatusFormatter,
     'verbose': VerboseStatusFormatter,
+    'json': JSONStatusFormatter,
     'urls': URLStatusFormatter,
     'ansi': AnsiStatusFormatter
 }
@@ -314,11 +358,11 @@ def get_formatter(action_type, options):
     if (not formatters_dict):
         raise TwitterError(
             "There was an error finding a class of formatters for your type (%s)"
-            %(action_type))
+            % (action_type))
     f = formatters_dict.get(options['format'])
     if (not f):
         raise TwitterError(
-            "Unknown formatter '%s' for status actions" %(options['format']))
+            "Unknown formatter '%s' for status actions" % (options['format']))
     return f()
 
 class Action(object):
@@ -333,7 +377,7 @@ class Action(object):
         if not careful:
             sample = '(Y/n)'
 
-        prompt = 'You really want to %s %s? ' %(subject, sample)
+        prompt = 'You really want to %s %s? ' % (subject, sample)
         try:
             answer = input(prompt).lower()
             if careful:
@@ -341,7 +385,7 @@ class Action(object):
             else:
                 return answer not in ('no', 'n')
         except EOFError:
-            print(file=sys.stderr) # Put Newline since Enter was never pressed
+            print(file=sys.stderr)  # Put Newline since Enter was never pressed
             # TODO:
                 #   Figure out why on OS X the raw_input keeps raising
                 #   EOFError and is never able to reset and get more input
@@ -371,7 +415,7 @@ class NoSuchActionError(Exception):
 
 class NoSuchAction(Action):
     def __call__(self, twitter, options):
-        raise NoSuchActionError("No such action: %s" %(options['action']))
+        raise NoSuchActionError("No such action: %s" % (options['action']))
 
 class StatusAction(Action):
     def __call__(self, twitter, options):
@@ -386,8 +430,8 @@ class SearchAction(Action):
     def __call__(self, twitter, options):
         # We need to be pointing at search.twitter.com to work, and it is less
         # tangly to do it here than in the main()
-        twitter.domain="search.twitter.com"
-        twitter.uriparts=()
+        twitter.domain = "search.twitter.com"
+        twitter.uriparts = ()
         # We need to bypass the TwitterCall parameter encoding, so we
         # don't encode the plus sign, so we have to encode it ourselves
         query_string = "+".join(
@@ -427,7 +471,7 @@ class ListsAction(StatusAction):
         screen_name = options['extra_args'][0]
 
         if not options['extra_args'][1:]:
-            lists = twitter.user.lists(user=screen_name)['lists']
+            lists = twitter.lists.list(screen_name=screen_name)
             if not lists:
                 printNicely("This user has no lists.")
             for list in lists:
@@ -448,15 +492,11 @@ class MyListsAction(ListsAction):
 
 class FriendsAction(StatusAction):
     def getStatuses(self, twitter, options):
-        return reversed(twitter.statuses.friends_timeline(count=options["length"]))
-
-class PublicAction(StatusAction):
-    def getStatuses(self, twitter, options):
-        return reversed(twitter.statuses.public_timeline(count=options["length"]))
+        return reversed(twitter.statuses.home_timeline(count=options["length"]))
 
 class RepliesAction(StatusAction):
     def getStatuses(self, twitter, options):
-        return reversed(twitter.statuses.replies(count=options["length"]))
+        return reversed(twitter.statuses.mentions_timeline(count=options["length"]))
 
 class FollowAction(AdminAction):
     def getUser(self, twitter, user):
@@ -477,7 +517,7 @@ class SetStatusAction(Action):
             s = ptr.match(statusTxt)
             if s and s.start() == 0:
                 replies.append(statusTxt[s.start():s.end()])
-                statusTxt = statusTxt[s.end()+1:]
+                statusTxt = statusTxt[s.end() + 1:]
             else:
                 break
         replies = " ".join(replies)
@@ -493,9 +533,11 @@ class SetStatusAction(Action):
                 end = string.rfind(statusTxt, ' ', 0, limit)
             else:
                 end = limit
-            splitted.append(" ".join((replies,statusTxt[:end])))
+            splitted.append(" ".join((replies, statusTxt[:end])))
             statusTxt = statusTxt[end:]
 
+        if options['invert_split']:
+            splitted.reverse()
         for status in splitted:
             twitter.statuses.update(status=status)
 
@@ -503,12 +545,12 @@ class TwitterShell(Action):
 
     def render_prompt(self, prompt):
         '''Parses the `prompt` string and returns the rendered version'''
-        prompt = prompt.strip("'").replace("\\'","'")
+        prompt = prompt.strip("'").replace("\\'", "'")
         for colour in ansi.COLOURS_NAMED:
-            if '[%s]' %(colour) in prompt:
+            if '[%s]' % (colour) in prompt:
                 prompt = prompt.replace(
-                    '[%s]' %(colour), ansi.cmdColourNamed(colour))
-        prompt = prompt.replace('[R]', ansi.cmdReset())
+                    '[%s]' % (colour), ansiFormatter.cmdColourNamed(colour))
+        prompt = prompt.replace('[R]', ansiFormatter.cmdReset())
         return prompt
 
     def __call__(self, twitter, options):
@@ -566,7 +608,7 @@ class RateLimitStatus(Action):
     def __call__(self, twitter, options):
         rate = twitter.account.rate_limit_status()
         print("Remaining API requests: %s / %s (hourly limit)" % (rate['remaining_hits'], rate['hourly_limit']))
-        print("Next reset in %ss (%s)" % (int(rate['reset_time_in_seconds']-time.time()),
+        print("Next reset in %ss (%s)" % (int(rate['reset_time_in_seconds'] - time.time()),
                                           time.asctime(time.localtime(rate['reset_time_in_seconds']))))
 
 actions = {
@@ -577,7 +619,6 @@ actions = {
     'mylist'    : MyListsAction,
     'help'      : HelpAction,
     'leave'     : LeaveAction,
-    'public'    : PublicAction,
     'pyprompt'  : PythonPromptAction,
     'replies'   : RepliesAction,
     'search'    : SearchAction,
@@ -594,6 +635,10 @@ def loadConfig(filename):
         for option in ('format', 'prompt'):
             if cp.has_option('twitter', option):
                 options[option] = cp.get('twitter', option)
+        # process booleans
+        for option in ('invert_split',):
+            if cp.has_option('twitter', option):
+                options[option] = cp.getboolean('twitter', option)
     return options
 
 def main(args=sys.argv[1:]):
@@ -601,7 +646,7 @@ def main(args=sys.argv[1:]):
     try:
         parse_args(args, arg_options)
     except GetoptError as e:
-        print("I can't do that, %s." %(e), file=sys.stderr)
+        print("I can't do that, %s." % (e), file=sys.stderr)
         print(file=sys.stderr)
         raise SystemExit(1)
 
@@ -614,12 +659,12 @@ def main(args=sys.argv[1:]):
     # arguments.
     options = dict(OPTIONS)
     for d in config_options, arg_options:
-        for k,v in list(d.items()):
+        for k, v in list(d.items()):
             if v: options[k] = v
 
     if options['refresh'] and options['action'] not in (
-        'friends', 'public', 'replies'):
-        print("You can only refresh the friends, public, or replies actions.", file=sys.stderr)
+        'friends', 'replies'):
+        print("You can only refresh the friends or replies actions.", file=sys.stderr)
         print("Use 'twitter -h' for help.", file=sys.stderr)
         return 1
 
@@ -631,13 +676,16 @@ def main(args=sys.argv[1:]):
             "the Command-Line Tool", CONSUMER_KEY, CONSUMER_SECRET,
             options['oauth_filename'])
 
+    global ansiFormatter
+    ansiFormatter = ansi.AnsiCmd(options["force-ansi"])
+
     oauth_token, oauth_token_secret = read_token_file(oauth_filename)
 
     twitter = Twitter(
         auth=OAuth(
             oauth_token, oauth_token_secret, CONSUMER_KEY, CONSUMER_SECRET),
         secure=options['secure'],
-        api_version='1',
+        api_version='1.1',
         domain='api.twitter.com')
 
     try:
